/*
 * @Author: your name
 * @Date: 2021-11-18 13:54:06
 * @LastEditTime: 2022-01-17 09:50:37
 * @LastEditors: Please set LastEditors
 * @Description: 打开koroFileHeader查看配置 进行设置:
 * https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
 * @FilePath: /EHR/home/igs/roadmodel/src/CtApRoadModel/source/road/frame.h
 */

/**
 * @file
 **/

#pragma once

#include <list>
#include <map>
#include <string>
#include <tuple>
#include <unordered_map>
#include <utility>
#include <vector>

#include "ilc_info.h"
#include "lane.h"
#include "lane_info.h"
#include "local_view.h"
#include "road_info_provider.h"
#include "status.h"
#include "vehicle_state_provider.h"

using rm::common::Status;

namespace rm {

/**
 * @class Frame
 * @brief Frame holds all data for one planning cycle.
 */
class Frame {
public:
  explicit Frame(uint32_t sequence_num);

  Frame(uint32_t sequence_num, const LocalView &local_view,
        RoadInfoProvider *road_info_provider);

  ~Frame() = default;

  /**
   * @brief: frame初始化 根据lane生成要加工的lane_info
   * @param {list<Lane>} &lanes
   * @return {*}
   */
  Status Init(const std::list<Lane> &lanes);
  /**
   * @brief: 读取序列号
   * @param {*}
   * @return {*}
   */
  uint32_t SequenceNum() const;
  /**
   * @brief: 返回lane_info地址
   * @param {*}
   * @return {*}
   */
  std::list<LaneInfo> *mutable_lane_info();
  /**
   * @brief: 返回road info
   * @param {*}
   * @return {*}
   */
  RoadInfoProvider *road_info_provider();
  /**
   * @brief: 返回车辆状态
   * @param {*}
   * @return {*}
   */
  // const VehicleStateProvider *vehicle_state_provider() const;
  /**
   * @brief: 返回local_view信息
   * @param {*}
   * @return {*}
   */
  const LocalView &local_view() const;

  uint8_t FindLaneIdFromLanePosition(const LanePosition &lane_position) {
    auto it = find(lane_info_.begin(), lane_info_.end(), lane_position);
    if (it == lane_info_.end())
      return 0;
    return it->mutable_lane()->lane_id();
  }

  Lane *FindLanePtrFromLanePosition(const LanePosition &lane_position) {
    auto it = find(lane_info_.begin(), lane_info_.end(), lane_position);
    if (it == lane_info_.end()) {
      return nullptr;
    }
    return it->mutable_lane();
  }

  LaneInfo *FindLaneInfoPtrFromLanePosition(const LanePosition &lane_position) {
    auto it = find(lane_info_.begin(), lane_info_.end(), lane_position);
    if (it == lane_info_.end()) {
      return nullptr;
    }
    return &(*it);
  }

  // the num of road edge with specific position could exceed one
  std::vector<RoadEdge *> FindRoadEdgePtrsFromRoadEdgePosition(
      const RoadEdgePostion &road_edge_position) {
    std::vector<RoadEdge *> road_edge_ptrs_list;
    std::for_each(road_info_provider_->mutable_road_edges()->begin(),
                  road_info_provider_->mutable_road_edges()->end(),
                  [&](RoadEdge &road_edge) {
                    if (road_edge.position() == road_edge_position) {
                      road_edge_ptrs_list.push_back(&road_edge);
                    }
                  });
    return road_edge_ptrs_list;
  }

  ILCInfo *mutable_ilc_info();

  void set_traffic_road_input_status(bool valid) {
    traffic_road_input_status_ = valid;
  }

  bool traffic_road_input_status() const { return traffic_road_input_status_; }

  void set_objects_input_status(bool valid) { objects_input_status_ = valid; }

  void set_is_straight_driving(bool flag) { is_straight_driving_ = flag; }

  bool is_straight_driving() const { return is_straight_driving_; }

  bool objects_input_status() const { return objects_input_status_; }

  void set_danger_pedestrain_id(uint8_t ped_id) {
    danger_pedestrain_id_ = ped_id;
  }

  uint8_t danger_pedestrain_id() const { return danger_pedestrain_id_; }

  rm::LaneLineCrossedType lane_line_crossed() const {
    return lane_line_crossed_;
  }

  void set_lane_line_crossed(uint8_t crossed_type) {
    lane_line_crossed_ = static_cast<rm::LaneLineCrossedType>(crossed_type);
  }

  bool ats_switch_flag() { return ats_switch_flag_; }

  void set_ats_switch_flag(bool ats_switch_flag) {
    ats_switch_flag_ = ats_switch_flag;
  }

  void set_ats_switch_state(uint8_t state) {
    ats_switch_state_ = state;
  }

  uint8_t ats_switch_state() {
    return ats_switch_state_;
  }

private:
  /**
   * @brief: TODO(all)更新交通等信息 障碍物信息
   * @param  {*}
   * @return {*}
   */
  common::Status InitFrameData();
  /**
   * @brief: 根据lane创建lane_info
   * @param {list<Lane>} &lanes
   * @return {*}
   */
  bool CreateLaneInfo(const std::list<Lane> &lanes);

private:
  uint32_t sequence_num_ = 0;                      //该frame的序列号
  LocalView local_view_;                           //输入信息
  RoadInfoProvider *road_info_provider_ = nullptr; //所有环境信息

  std::list<LaneInfo> lane_info_;
  ILCInfo ilc_info_;
  bool traffic_road_input_status_;
  bool objects_input_status_;
  bool is_straight_driving_;
  uint8_t danger_pedestrain_id_ = 0U;
  LaneLineCrossedType lane_line_crossed_;
  bool ats_switch_flag_;
  uint8_t ats_switch_state_;
};

} // namespace rm
